var localDirTree = {
  data : new Array,
  rowCount : 0,
  getCellText : function(row,column) {    // names for the directories
    return row == -1 ? "" : this.data[row].leafName;
  },
  getLevel : function(row) {              // level is based on number of slashes in the path
    if (row <= 0 || row >= this.data.length)
      return 0; 
    return this.data[row].path().match(slash == "/" ? /\x2f/g : /\x5c/g ).length;
  },
  getParentIndex : function(row) {
    if (row <= 0)
      return -1;
    return this.data[row].parentIndex;
  },
  getImageSrc         : function(row,col)         {},
  getCellProperties   : function(row,col,props)   {},
  getColumnProperties : function(colid,col,props) {},
  getRowProperties    : function(row,props)       {},
  hasNextSibling      : function(row,nextrow)     { return this.data[row].hasNext; },
  isContainer         : function(row)             { return true; },
  isContainerEmpty    : function(row)             { return this.data[row].empty; },
  isContainerOpen     : function(row)             { return this.data[row].open; },
  isSeparator         : function(row)             { return false; },
  isSorted            : function(row)             { return false; },
  setTree             : function(treebox)         { this.treebox = treebox; },
  toggleOpenState     : function(row)             {
    if (this.isContainerOpen(row)) {
      var currentIndex = this.selection.count ? this.selection.currentIndex : -1;
      
      var level = this.getLevel(row);
      var lastChild = row;
      while (lastChild + 1 < this.rowCount && this.getLevel(lastChild + 1) > level)    // find last index in same level as collapsed dir  
        ++lastChild;
      this.data[row].children = this.data.splice(row + 1, lastChild - row);            // get rid of subdirectories from view
      this.updateParentIndices();
      this.rowCount = this.data.length;
      this.treebox.rowCountChanged(row, -(lastChild - row));

      this.data[row].open = false;
      this.treebox.invalidateRow(row);                                                 // update row

      if (gLocalPath.value.indexOf(this.data[row].path()) == 0
          && gLocalPath.value != this.data[row].path()
          && gLocalPath.value.match(slash == "/" ? /\x2f/g : /\x5c/g ).length > this.getLevel(row)) {  // we were in a subdirectory and we collapsed
        gLocalPath.value = this.data[row].path();
        localTree.updateView();
        this.selection.select(row);
        this.treebox.ensureRowIsVisible(row);
      } else if (gLocalPath.value == this.data[row].path()) {
        this.selection.select(row);
        this.treebox.ensureRowIsVisible(row);
      }
    } else {
      if (this.data[row].children) {                                                   // stored from before
        for (var x = this.data[row].children.length - 1; x >= 0; --x)
          this.data.splice(row + 1, 0, this.data[row].children[x]);
        this.updateParentIndices();
        this.rowCount = this.data.length;
        this.treebox.rowCountChanged(row + 1, this.data[row].children.length);
        this.data[row].children = null;
        this.data[row].open = true;
        this.treebox.invalidateRow(row);
      } else {                                                                         // get data for this directory
        var newDirectories = new Array;
        
        try {
          var dir = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
          dir.initWithPath(this.data[row].path());
          var entries = dir.directoryEntries;
          while (entries.hasMoreElements()) {
            var file = entries.getNext().QueryInterface(Components.interfaces.nsILocalFile);  // get subdirectories
            if (file.exists() && file.isDirectory() && (!file.isHidden() || hiddenMode))
              newDirectories.push(file);
          }
        } catch (ex) {
          error(strbundle.getString("noPermission"));
          debug(ex,31);
        }

        if (newDirectories.length == 0)  {                                             // no subdirectories
          this.data[row].empty = true;
          this.data[row].open = false;
        } else {                                                                       // has subdirectories
          for (var x = 0; x < newDirectories.length; ++x) {
            newDirectories[x] = { leafName: newDirectories[x].leafName, parent: newDirectories[x].parent ? newDirectories[x].parent.path: "",
                        path: function() { return localTree.constructPath(this.parent, this.leafName); },
                        open: false, empty: false, children: null, hasNext: true, parentIndex: -1 };
          }

          newDirectories.sort(directorySort);
          newDirectories[newDirectories.length - 1].hasNext = false;                   // last one doesn't have a next
          for (var x = newDirectories.length - 1; x >= 0; --x)
            this.data.splice(row + 1, 0, newDirectories[x]);
          this.updateParentIndices();
          this.rowCount = this.data.length;
          this.treebox.rowCountChanged(row + 1, newDirectories.length);
          this.data[row].open = true;
        }
        this.treebox.invalidateRow(row);
      }
    }
  },
  updateParentIndices : function () {
    for (var x = 0; x < this.data.length; ++x) {
      var pIndex = this.data[x].parent ? this.indexOfPath(this.data[x].parent) : -1;   // ah, beautiful
      this.data[x].parentIndex = pIndex;
    }
  },

  // ************************************************* mouseEvent *****************************************************

  click : function(event) {
    if (event.button == 0 || event.button == 2) {
      var row = {};    var col = {};    var child = {};
      this.treebox.getCellAt(event.pageX, event.pageY, row, col, child);
      var index = this.selection.currentIndex;

      if (index == row.value && this.data[index].path() != gLocalPath.value)           // index == row.value in case were collapsing the folder
        this.changeDir(this.data[index].path());
    }
  },

  // ************************************************* keyEvent *****************************************************

  localDirKeyDownIndex: -1,
  localDirKeyDownOpen : -1,
  keyDown  : function(event) {
    this.localDirKeyDownIndex = this.selection.currentIndex;
    this.localDirKeyDownOpen = this.data[this.localDirKeyDownIndex].open;
  },
  keyPress : function(event) {
    var index = this.selection.currentIndex;
    switch (event.keyCode) {
      case 37:                    // left arrow
        if (!this.data[index].open && this.getParentIndex(index) != -1) {
          gLocalPath.value = this.data[this.getParentIndex(index)].path();
          localTree.updateView();
          this.selection.select(index);
        }
        break;
      case 38:                    // up arrow
      case 40:                    // down arrow
        if (event.keyCode == 38 && !index)
          return;
        if (event.keyCode == 40 && index == this.data.length - 1) {
          this.selection.select(index - 1);    // this shouldn't work but it does
          return;
        }
        var x = event.keyCode == 38 ? -1 : 1;

        // see if file is good first
        try {
          dir = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
          dir.initWithPath(this.data[index + x].path());
          var entries = dir.directoryEntries;
        } catch (ex) {
          debug(ex,28);
          this.changeDir(this.data[index + x].path());
          this.selection.select(index);
          break;
        }

        gLocalPath.value = this.data[index + x].path();
        localTree.updateView();
        this.selection.select(index);
        break;
      case 39:                    // right arrow
        if (this.localDirKeyDownOpen && !this.data[index].empty) {

          // see if file is good first
          try {
            dir = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
            dir.initWithPath(this.data[index + 1].path());
            var entries = dir.directoryEntries;
          } catch (ex) {
            error(strbundle.getString("noPermission"));
            debug(ex,30);
            break;
          }

          gLocalPath.value = this.data[index + 1].path();
          localTree.updateView();
        }
        break;
      case 8:                     // backspace
        this.cdup();
        break;
      case 116:                   // F5
        event.preventDefault();
        localTree.refresh();
        break;
    }
  },

  // ********************************************** file functions ***************************************************

  getFile : function(row) {                                                          // get file based on path
    var file = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(this.data[row].path());
    return file;
  },
  indexOfPath : function(path) {                                                     // binary search to find a path in the localDirTree
    var left = 0;
    var right = this.data.length - 1;
    path = path.replace(slash == "\\" ? /\x5c/g : /\x2f/g, "\x01").toLowerCase();    // make '/' less than everything (except null really)

    while (left <= right) {
      var mid = Math.floor((left + right) / 2);
      var dataPath = this.data[mid].path().replace(slash == "\\" ? /\x5c/g : /\x2f/g, "\x01").toLowerCase();
      if (dataPath == path || dataPath + slash == path || dataPath == path + slash)  // kind of complicated but what can you do
        return mid;
      else if (path < dataPath)
        right = mid - 1;
      else if (path > dataPath)
        left = mid + 1;
    }
    return -1;
  },
  reselectCurrentDirectory : function() {
    var index = this.indexOfPath(gLocalPath.value);                                  // reselect the current directory
    this.selection.select(index);
    this.treebox.ensureRowIsVisible(index);
  },
            
  // ************************************************* cdup *****************************************************

  cdup : function() {
    var parentIndex = this.getParentIndex(this.selection.currentIndex);
    if (parentIndex != -1) {
      this.changeDir(this.data[parentIndex].path());
      this.selection.select(parentIndex);
    }
  },

  // ************************************************* change directories *****************************************************

  changeDir : function (path) {
    gLocalPath.value = path;

    if (this.data.length == 0 || this.data[0].path().charAt(0) != gLocalPath.value.charAt(0)) {    // if dirTree is empty
      var oldRowCount = this.rowCount;                                                             // or we're switching to a new drive
      this.data = new Array;
      this.rowCount = 0;
      this.treebox.rowCountChanged(0, -oldRowCount);

      var thepath;                                                                                 // we restart the tree
      if (gLocalPath.value.indexOf('/') == 0) {                                                    // linux
        thepath = "/";
        slash = "/";
      } else {                                                                                     // windows
        if (gLocalPath.value.indexOf('\\') == -1) {
          gLocalPath.value += "\\";
        }
        thepath = gLocalPath.value.substring(0, gLocalPath.value.indexOf('\\') + 1);
        slash = "\\";
      }

      this.data.push( {leafName: thepath, parent: "",
                   path: function() { return this.leafName; },
                   open: false, empty: false, children: null, hasNext:false, parentIndex: -1} );
      this.rowCount = 1;
      this.treebox.rowCountChanged(0, 1);
      this.selection.select(0);
    }

    if (slash == "/")                                                                              // error checking here for correct values in path
      gLocalPath.value = gLocalPath.value.replace(/\x5c/g, "/");                                   // linux shouldn't have backslashes
    else
      gLocalPath.value = gLocalPath.value.replace(/\x2f/g, "\\");                                  // windows shouldn't have forward slashes

    if (gLocalPath.value != '/' && gLocalPath.value.charAt(gLocalPath.value.length - 1) == slash)  // trim slashes at the end - ruins levels in dir
      gLocalPath.value = gLocalPath.value.substring(0, gLocalPath.value.length - 1);
    if (slash == "\\" && gLocalPath.value.indexOf('\\') == -1)                                     // if it's windows we add a slash to it: 'c:'->'c:\'
      gLocalPath.value += "\\";
    if (slash == "/" && gLocalPath.value.charAt(0) != '/')                                         // linux path has to start with '/'
      gLocalPath.value = '/' + gLocalPath.value;

    var bestMatch;
    var localPathLevel = gLocalPath.value.match(slash == "/" ? /\x2f/g : /\x5c/g ).length;
    for (var x = 0; x < this.data.length; ++x) {                                                   // open parent directories til we find the directory
      for (var y = this.data.length - 1; y >= x; --y)
        if (gLocalPath.value.indexOf(this.data[y].path()) == 0
            && (this.getLevel(y) < localPathLevel || gLocalPath.value == this.data[y].path())) {
          x = y;
          bestMatch = x;
          break;
        }

      if (gLocalPath.value.indexOf(this.data[x].path()) == 0) {
        if (!this.data[x].open)
          this.toggleOpenState(x);
        if (gLocalPath.value == this.data[x].path()) {
          localPathFocus = path;                                                                   // directory approved
          var gFormHistory = Components.classes["@mozilla.org/satchel/form-history;1"]             // add directory to drop down list
                                       .getService(Components.interfaces.nsIFormHistory);
          gFormHistory.addEntry(gLocalPath.getAttribute("autocompletesearchparam"), gLocalPath.value);

          savePreferences();                                                                       // remember last directory

          localTree.updateView();
          return;
        }
      }
    }

    var findDirectory = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
    findDirectory.initWithPath(gLocalPath.value);                                                  // we didn't find the directory
    if (localFile.verifyExists(findDirectory)) {
      var tempPath = gLocalPath.value;
      this.selection.select(bestMatch);                                                            // it's possible the directory was added externally
      localTree.refresh(true);                                                                     // and we don't have it on our dir list
      this.changeDir(tempPath);
    }
  },
  canDrop : function (aIndex, aOrientation) { return false; }
};
