var localFile = {
  launch : function(file) {
    try {
      if (file.exists())
        file.launch();
    } catch (ex) { debug(ex,37); }
  },
  createDirectory : function() {
    var name = window.prompt(strbundle.getString("directoryName"), "", strbundle.getString("newDirectory"));
    if (!name)
      return null;

    var dir = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
    dir.initWithPath(localTree.constructPath(gLocalPath.value, name));
    try {
      dir.create(Components.interfaces.nsILocalFile.DIRECTORY_TYPE, 0755);              // create the directory
    } catch (ex) {
      error(strbundle.getString("dirFail"));
      debug(ex,33);
      return null;
    }
    return dir;
  },
  remove : function(file, prompt, multiple) {
    if (prompt && multiple && multiple > 1) {
      if (!window.confirm(strbundle.getString("confirmDelete") + " " + multiple + " " + strbundle.getString("confirmDelete2")
             + '\n' + strbundle.getString("localDeleteWarning")))
        return false;
    } else if (prompt && file.isDirectory()) {                                          // deleting a directory
      if (!window.confirm(strbundle.getString("confirmDelete") + " '" + file.leafName + "' " + strbundle.getString("confirmDelete3")
             + '\n' + strbundle.getString("localDeleteWarning")))
        return false;
    } else if (prompt) {                                                                // deleting a file
      if (!window.confirm(strbundle.getString("confirmDelete") + " '" + file.leafName + "'?" + '\n' + strbundle.getString("localDeleteWarning")))
        return false;
    }

    try {
      file.remove(true);                                                                // delete file (recursively if dir)
    } catch (ex) {
      error(strbundle.getString("delFail"));
      debug(ex,34);
      return false;
    }
    return true;
  },
  rename : function(file) {
    if (!file.exists())                                                                 // check it exists
      return false;

    var newName = window.prompt(strbundle.getString("renameTo"), file.leafName, strbundle.getString("rename"));
    if (!newName || file.leafName == newName)
      return false;

    var oldName = file.leafName;

    try {
      var newFile = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
      newFile.initWithPath(file.parent.path + slash + newName);
      if (newFile && newFile.exists() && (slash == '/' || oldName.toLowerCase() != newName.toLowerCase())) {
        var params = { response         : 0,
                       fileName         : newName,
                       resume           : true,
                       replaceResume    : true,
                       timerEnable      : false,
                       doingRename      : true };

        dialogOpen = true;
        window.openDialog("chrome://fireftp/content/confirmFile.xul","confirmFile",
          "width=425,height=90,chrome,modal=yes,dialog=yes,resizable=yes,centerscreen", params);
        dialogOpen = false;

        if ((params.response == 4) || (params.response == 0))
          return false;
      }

      file.moveTo(null, newName);                                                       // rename the file
    } catch (ex) {
      if (slash == '\\' && oldName.toLowerCase() == newName.toLowerCase())              // we renamed the file the same but with different case
        return true;                                                                    // for some reason this throws an exception

      error(strbundle.getString("renameFail"));
      debug(ex,32);
      return false;
    }
    return true;
  },
  showProperties : function(file, recursive) {
    try {
      var date = new Date(file.lastModifiedTime);
      date = months[date.getMonth()] + ' ' + date.getDate() + ' ' + date.getFullYear() + ' ' + date.toLocaleTimeString();

      var recursiveFolderData = { type: "local", nFolders: 0, nFiles: 0, nSize: 0 };
      if (file.isDirectory() && recursive)
        localTree.getRecursiveFolderData(file, recursiveFolderData);

      var origWritable = file.isWritable();

      var params = { path                : file.path,
                     leafName            : file.leafName,
                     fileSize            : file.fileSize,
                     date                : date,
                     origPermissions     : slash == "/" ? "-" + localTree.convertPermissions(false, file.permissions) : 0,
                     permissions         : "",
                     writable            : file.isWritable(),
                     hidden              : file.isHidden(),
                     isDirectory         : file.isDirectory(),
                     multipleFiles       : false,
                     isLinuxType         : slash == "/",
                     isLocal             : true,
                     recursiveFolderData : file.isDirectory() && recursive ? recursiveFolderData : null,
                     returnVal           : false };

      dialogOpen = true;
      window.openDialog("chrome://fireftp/content/properties.xul","properties",
        "width=600,height=360,chrome,modal=yes,dialog=yes,resizable=yes,centerscreen", params);
      dialogOpen = false;

      if (!params.returnVal)
        return false;

      if (params.isLinuxType) {
        file.permissions = parseInt(params.permissions, 8);
        return true;
      } else if (origWritable != params.writable) {
        if (params.writable)
          file.permissions = file.permissions == 365 ? 511 : 438;
        else
          file.permissions = file.permissions == 511 ? 365 : 292;
        return true;
      }
    } catch (ex) { debug(ex,41); }
    return false;
  },
  verifyExists : function(file) {
    var exists = file && file.exists();
    if (!exists)
      error("'" + file.leafName + "' " + strbundle.getString("fileDoesNotExist"));
    return exists;
  }
}
