var remoteTree = {
  data        : new Array,
  rowCount    : 0,
  remoteSize  : 0,
  getCellText : function(row,column) {    // text for the files
    if (row == -1)
      return "";
    switch(column.id) {
      case "remotename":
        return this.data[row].leafName;
      case "remotesize":
        if (this.data[row].fileSize == '0')
          return "0" + (bytesMode ? "  " : " KB  ");
        if (bytesMode)
          return this.data[row].fileSize + "  ";
        else
          return commas(Math.floor(this.data[row].fileSize / 1024) + 1) + " KB  ";
      case "remotedate":
        return this.data[row].date;
      case "remotetype":
        return this.getExtension(this.data[row].leafName);
      case "remoteattr":
        return this.data[row].permissions;
      default:
        return " "; // shouldn't get here
    }
  },
  getExtension : function(leafName) {
    if (leafName.lastIndexOf(".") != -1)
      return leafName.substring(leafName.lastIndexOf(".") + 1, leafName.length).toLowerCase();
    else
      return "";
  },
  getImageSrc : function(row,col) {    // icons for the files
    if (row == -1)
      return "";
    if (col.id == "remotename") {
      if (this.data[row].isDirectory || this.data[row].isSymlink)
        return "";
      return "moz-icon://" + this.data[row].leafName + "?size=16";
    }
  },
  cycleHeader : function(col) {
    var sortDirection = col.element.getAttribute("sortDirection") == "ascending" || col.element.getAttribute("sortDirection") == "natural"
                        ? "descending" : "ascending";
    document.getElementById('remotename').setAttribute("sortDirection", "natural");
    document.getElementById('remotesize').setAttribute("sortDirection", "natural");
    document.getElementById('remotedate').setAttribute("sortDirection", "natural");
    document.getElementById('remotetype').setAttribute("sortDirection", "natural");
    document.getElementById('remoteattr').setAttribute("sortDirection", "natural");
    col.element.setAttribute("sortDirection", sortDirection);
    this.sort();
  },
  getCellProperties : function(row, col, props)   {
    if (row == -1)
      return;
    if (col.id == "remotename") {
      if (this.data[row].isDirectory)
        props.AppendElement(gAtomService.getAtom("isFolder"));
      else if (this.data[row].isSymlink)
        props.AppendElement(gAtomService.getAtom("isLink"));
    }
  },
  getParentIndex      : function(row)             { return -1; },
  getLevel            : function(row)             { return 0; },
  getRowProperties    : function(row,props)       {},
  getColumnProperties : function(colid,col,props) {},
  isContainer         : function(row)             { return false; },
  isSeparator         : function(row)             { return false; },
  isSorted            : function(row)             { return false; },
  setTree             : function(treebox)         { this.treebox = treebox; },
  
  // ************************************************* mouseEvent *****************************************************

  dblClick : function(event) {
    if (!isConnected || event.button != 0 || event.originalTarget.localName != "treechildren" || this.selection.count == 0)
      return;

    if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
      this.selection.currentIndex = this.rowCount - 1;

    if (this.data[this.selection.currentIndex].isDirectory) {         // if it's a directory
      if (eventQueue.length)
        return;
      remoteDirTree.changeDir(this.constructPath(gRemotePath.value, this.data[this.selection.currentIndex].leafName));    // navigate to it
    } else if (this.data[this.selection.currentIndex].isSymlink) {    // if it's a symbolic link
      if (checkIsListing())
        return;

      var linkedFile = this.data[this.selection.currentIndex].symlink;
      var parentPath = gRemotePath.value;
      while (linkedFile.indexOf("./") == 0 || linkedFile.indexOf("../") == 0) {
        if (linkedFile.indexOf("./") == 0)
          linkedFile = linkedFile.substring(2);
        else {
          linkedFile = linkedFile.substring(3);
          parentPath = parentPath.substring(0, parentPath.lastIndexOf('/') ? parentPath.lastIndexOf('/') : 1);
        }
      }

      if (linkedFile.indexOf("/") != 0)
        linkedFile = this.constructPath(parentPath, linkedFile);

      addEventQueue("CWD", linkedFile);
      if (isReady)                                                    // get the ball rollin' if it isn't already
        writeControl(eventQueue[0].cmd, eventQueue[0].parameter);
    } else
      transfer(true);                                                 // else download the file
  },
  click : function(event) {
    if (!isConnected)
      return;
    if (event.button == 1 && !document.getElementById('remotePasteContext').disabled)   // middle-click paste
      this.paste();
  },
  mouseOver : function(event) {                                       // display remote folder info
    if (this.rowCount)
      document.getElementById('statustxt').label =
        strbundle.getString("remoteListing") + " " + this.rowCount + " " + strbundle.getString("objects") + " " + commas(this.remoteSize);
    else
      document.getElementById('statustxt').label = strbundle.getString("remoteListingNoObjects");
  },

  // ************************************************* keyEvent *****************************************************

  keyPress : function(event) {
    if (!isConnected)
      return;

    if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
      this.selection.currentIndex = this.rowCount - 1;

    if (event.keyCode == 13 && this.selection.count != 0) {                                         // enter
      if (this.selection.count == 1 && this.data[this.selection.currentIndex].isDirectory) {        // if it's a directory
        if (eventQueue.length)
          return;

        remoteDirTree.changeDir(this.constructPath(gRemotePath.value, this.data[this.selection.currentIndex].leafName));    // navigate to it
      } else
        transfer(true);                                                                             // else retrieve a file
    } else if (event.ctrlKey && (event.which == 65 || event.which == 97)) {
      event.preventDefault();                                                                       // ctrl-a: select all
      this.selection.selectAll();
    } else if (event.ctrlKey && event.which == 32 && this.selection.count != 0)                     // ctrl-space, select or deselect
      this.selection.toggleSelect(this.selection.currentIndex);
    else if (event.keyCode == 8) {                                                                  // backspace
      event.preventDefault();
      remoteDirTree.cdup();
    } else if (event.keyCode == 116) {                                                              // F5
      event.preventDefault();
      this.refresh();
    } else if (event.keyCode == 113 && this.selection.count != 0)                                   // F2
      this.rename();
    else if (event.charCode == 109 && event.ctrlKey) {                                              // ctrl-m
      event.preventDefault();
      this.createDirectory();
    } else if (event.keyCode == 46 && this.selection.count != 0)                                    // del
      this.remove();
    else if (event.keyCode == 93) {                                                                 // display context menu
      var x = {};    var y = {};    var width = {};    var height = {};  
      this.treebox.getCoordsForCellItem(this.selection.currentIndex, this.treebox.columns["remotename"], "text", x, y, width, height);
      document.getElementById('remotemenu').showPopup(
          gRemoteTreeChildren, gRemoteTreeChildren.boxObject.x + 75, gRemoteTreeChildren.boxObject.y + y.value + 5, "context");
    } else if (event.charCode == 112 && event.ctrlKey && this.selection.count != 0) {               // ctrl-p
      event.preventDefault();
      this.showProperties(false);
    } else if (event.charCode == 120 && event.ctrlKey && this.selection.count != 0) {               // ctrl-x
      event.preventDefault();
      this.cut();
    } else if (event.charCode == 118 && event.ctrlKey) {                                            // ctrl-v
      event.preventDefault();
      this.paste();
    } else if (event.charCode == 111 && event.ctrlKey) {                                            // ctrl-o
      event.preventDefault();
      this.viewOnTheWeb();
    }
  },

  // ************************************************* file functions ***************************************************

  constructPath : function(parent, leafName) {
    return parent + (parent.charAt(parent.length - 1) != '/' ? '/' : '') + leafName;
  },
  viewOnTheWeb : function() {
    if (!isConnected || this.selection.count == 0)
      return;
    if (!webHost) {
      window.openDialog("chrome://fireftp/content/alert.xul","alert",
        "width=400,height=200,chrome,modal=yes,dialog=yes,resizable=yes,centerscreen", strbundle.getString("fillInWebHost"));
      return;
    }
    if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
      this.selection.currentIndex = this.rowCount - 1;

    var path = this.constructPath(gRemotePath.value, this.data[this.selection.currentIndex].leafName);

    runInFirefox(webHost + path);
  },
  createDirectory : function() {
    if (!isConnected || eventQueue.length)
      return;
    var dir = window.prompt(strbundle.getString("directoryName"), "", strbundle.getString("newDirectory"));
    if (!dir)
      return;

    addEventQueue("MKD", this.constructPath(gRemotePath.value, dir), refreshMode ? "remoteTree.refresh()" : "");
    if (isReady)
      writeControl(eventQueue[0].cmd, eventQueue[0].parameter);
  },
  remove : function() {
    if (!isConnected || eventQueue.length || this.selection.count == 0
      || !window.confirm(strbundle.getString("confirmDelete") + " " + gRemoteTree.view.selection.count + " " + strbundle.getString("confirmDelete2")))
      return;

    var last = true;
    for (var x = 0; x < this.rowCount; ++x)
      if (this.selection.isSelected(x)) {
        var remotePath = this.constructPath(gRemotePath.value, this.data[x].leafName);
        if (this.data[x].isDirectory) {                         // delete a subdirectory recursively
          unshiftEventQueue("RMD", remotePath, (last && refreshMode) ? "remoteTree.refresh()" : "");
          list(remotePath, "remoteTree.removeRecursive(\'" + remotePath + "\')", true, true);// get list of subfiles
        } else                                                  // delete a file
          eventQueue.unshift( { cmd: "DELE", parameter: remotePath, remember: (last && refreshMode) ? "remoteTree.refresh()" : "" } );
        last = false;
      }
    if (isReady)
      writeControl(eventQueue[0].cmd, eventQueue[0].parameter);
  },
  removeRecursive : function(parent) {                          // delete subdirectories and files
    var files = data;
    
    for (var x = 0; x < files.length; ++x) {
      var remotePath = this.constructPath(parent, files[x].leafName);

      if (files[x].isDirectory) {                               // delete a subdirectory recursively
        unshiftEventQueue("RMD", remotePath, "");
        list(remotePath, "remoteTree.removeRecursive(\'" + remotePath + "\')", true, true);
      } else                                                    // delete a file
        unshiftEventQueue("DELE", remotePath, "");
    }
  },
  rename : function() {
    if (!isConnected || this.rowCount == 0 || eventQueue.length || this.selection.count == 0)
      return;

    if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
      this.selection.currentIndex = this.rowCount - 1;

    var path = this.constructPath(gRemotePath.value, this.data[this.selection.currentIndex].leafName);

    var newName = window.prompt(strbundle.getString("renameTo"), path.substring(path.lastIndexOf('/') + 1), strbundle.getString("rename"));
    if (!newName)
      return;
    
    for (var x = 0; x < this.rowCount; ++x)
      if (this.data[x].leafName == newName) {
        var params = { response         : 0,
                       fileName         : newName,
                       resume           : true,
                       replaceResume    : true,
                       timerEnable      : false,
                       doingRename      : true };

        dialogOpen = true;
        window.openDialog("chrome://fireftp/content/confirmFile.xul","confirmFile",
          "width=425,height=90,chrome,modal=yes,dialog=yes,resizable=yes,centerscreen", params);
        dialogOpen = false;

        if ((params.response == 4) || (params.response == 0))
          return;
      }

    if (path.charAt(path.length - 1) == '/')
      path = path.substring(path.length - 1);
    newName = path.substring(0, path.lastIndexOf('/')) + '/' + newName;
    
    addEventQueue("RNFR", path);                                // rename the file
    addEventQueue("RNTO", newName, refreshMode ? "remoteTree.refresh()" : "");
    if (isReady)
      writeControl(eventQueue[0].cmd, eventQueue[0].parameter);
  },
  showProperties : function(recursive) {
    if (this.selection.count == 0 || this.rowCount == 0)
      return;

    if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
      this.selection.currentIndex = this.rowCount - 1;

    this.recursiveFolderData = { type: "remote", nFolders: 0, nFiles: 0, nSize: 0, files: new Array() };
    if (this.selection.count > 1) {                             // multiple files
      var last = true;
      for (var x = 0; x < this.rowCount; ++x)
        if (this.selection.isSelected(x)) {
          if (this.data[x].isDirectory) {
            ++this.recursiveFolderData.nFolders;
            if (recursive) {
              var remotePath = this.constructPath(gRemotePath.value, this.data[x].leafName);
              list(remotePath, "remoteTree.getRecursiveFolderData(\'" + remotePath + "\'," + last + ")", true, true);
              last = false;
            }
          } else
            ++this.recursiveFolderData.nFiles;
          this.recursiveFolderData.files.push(this.data[x]);
          this.recursiveFolderData.nSize += parseInt(this.data[x].fileSize);
        }
      if (last)
        this.showMultipleProperties();
      else if (isReady)
        writeControl(eventQueue[0].cmd, eventQueue[0].parameter);

      return;
    }

    var index = this.selection.currentIndex;

    var thePath = this.constructPath(gRemotePath.value, this.data[index].leafName);

    if (this.data[index].isDirectory && recursive) {            // directory
      list(thePath, "remoteTree.getRecursiveFolderData(\'" + thePath + "\', true, true)", true, true);
      this.recursiveFolderData.nSize += parseInt(this.data[index].fileSize);
      if (isReady)
        writeControl(eventQueue[0].cmd, eventQueue[0].parameter);
      return;
    }

    var params = { path            : thePath,
                   leafName        : this.data[index].leafName,
                   fileSize        : this.data[index].fileSize,
                   date            : this.data[index].date,
                   origPermissions : this.data[index].permissions,
                   writable        : 'disabled',
                   hidden          : 'disabled',
                   isDirectory     : this.data[index].isDirectory,
                   user            : this.data[index].user,
                   group           : this.data[index].group,
                   permissions     : "",
                   webHost         : webHost,
                   isSymlink       : this.data[index].isSymlink,
                   symlink         : this.data[index].symlink };

    dialogOpen = true;
    window.openDialog("chrome://fireftp/content/properties.xul","properties",
      "width=600,height=360,chrome,modal=yes,dialog=yes,resizable=yes,centerscreen", params);
    dialogOpen = false;

    if (params.permissions && isConnected && eventQueue.length == 0) {                      // permissions were changed; CHMOD!
      addEventQueue("SITE CHMOD", params.permissions + ' ' + thePath, refreshMode ? "remoteTree.refresh()" : "");
      if (isReady)
        writeControl(eventQueue[0].cmd, eventQueue[0].parameter);
    }
  },
  recursiveFolderData : new Object(),
  getRecursiveFolderData : function(parent, last, showDir) {
    var files = data;
    
    for (var x = 0; x < files.length; ++x) {
      var remotePath = this.constructPath(parent, files[x].leafName);

      if (files[x].isDirectory) {
        ++this.recursiveFolderData.nFolders;
        list(remotePath, "remoteTree.getRecursiveFolderData(\'" + remotePath + "\'," + last + "," + showDir + ")", true, true);
        last = false;
      } else
        ++this.recursiveFolderData.nFiles;
      this.recursiveFolderData.files.push(files[x]);
      this.recursiveFolderData.nSize += parseInt(files[x].fileSize);
    }
    if (last)
      this.showMultipleProperties(showDir);
  },
  showMultipleProperties : function(dir) {
    var params;
    var thePath;

    dialogOpen = true;
    if (dir) {
      var index = this.selection.currentIndex;
      thePath = this.constructPath(gRemotePath.value, this.data[index].leafName);

      params = { path                : thePath,
                 leafName            : this.data[index].leafName,
                 fileSize            : 0,
                 date                : this.data[index].date,
                 origPermissions     : this.data[index].permissions,
                 writable            : 'disabled',
                 hidden              : 'disabled',
                 isDirectory         : this.data[index].isDirectory,
                 user                : this.data[index].user,
                 group               : this.data[index].group,
                 permissions         : "",
                 isSymlink           : this.data[index].isSymlink,
                 symlink             : this.data[index].symlink,
                 multipleFiles       : false,
                 recursiveFolderData : this.recursiveFolderData,
                 applyTo             : { type: "remote", thisFile: true, folders: false, files: false } };

      window.openDialog("chrome://fireftp/content/properties.xul","properties",
        "width=600,height=360,chrome,modal=yes,dialog=yes,resizable=yes,centerscreen", params);
    } else {
      params = { multipleFiles       : true,
                 recursiveFolderData : this.recursiveFolderData,
                 permissions         : "",
                 applyTo             : { folders: false, files: true } };

      window.openDialog("chrome://fireftp/content/properties.xul","properties",
        "width=600,height=360,chrome,modal=yes,dialog=yes,resizable=yes,centerscreen", params);
    }
    dialogOpen = false;

    if (params.permissions && isConnected) {                      // permissions were changed; CHMOD!
      var last = true;
      if (dir && params.applyTo.thisFile) {
        unshiftEventQueue("SITE CHMOD", params.permissions + ' ' + thePath, (last && refreshMode) ? "remoteTree.refresh()" : "");
        last = false;
      }

      for (var x = 0; x < this.recursiveFolderData.files.length; ++x) {
        if ((this.recursiveFolderData.files[x].isDirectory && params.applyTo.folders)
            || (!this.recursiveFolderData.files[x].isDirectory && params.applyTo.files)) {
          unshiftEventQueue("SITE CHMOD", params.permissions + ' ' + this.recursiveFolderData.files[x].path,
                            (last && refreshMode) ? "remoteTree.refresh()" : "");
          last = false;
        }
      }

      if (isReady && eventQueue.length)
        writeControl(eventQueue[0].cmd, eventQueue[0].parameter);
    }
  },

  // ****************************************************** refresh ***************************************************

  refresh : function() {
    this.data = null;                                                               // clear remoteTree to make it look like refreshing
    this.treebox.rowCountChanged(0, -this.rowCount);
    this.rowCount = 0;

    if (remoteDirTree.data[remoteDirTree.selection.currentIndex].empty)             // if remoteDirTree is empty
      remoteDirTree.data[remoteDirTree.selection.currentIndex].empty = false;       // it's not anymore

    if (remoteDirTree.data[remoteDirTree.selection.currentIndex].open) {            // if remoteDirTree is open
      remoteDirTree.toggleOpenState(remoteDirTree.selection.currentIndex);          // close it up
      remoteDirTree.data[remoteDirTree.selection.currentIndex].children = null;     // reset its children
      removeCacheEntry(remoteDirTree.data[remoteDirTree.selection.currentIndex].path());    // clear out cache entry
      remoteDirTree.updateViewAfter = true;                                         // refresh remoteTree afterwards
      remoteDirTree.toggleOpenState(remoteDirTree.selection.currentIndex);          // and open it up again
    } else
      this.updateView(true);
  },

  // ****************************************************** updateView ***************************************************

  updateView : function(skipCache) {
    list(gRemotePath.value, "remoteTree.updateView2()", skipCache);
  },
            
  updateView2 : function(column, direction) {
    var remoteTreeItems = data;

    this.remoteSize = 0;                                                            // get directory size
    for (var x = 0; x < remoteTreeItems.length; ++x)
      this.remoteSize += parseInt(remoteTreeItems[x].fileSize); 
    this.remoteSize = parseSize(this.remoteSize);

    this.data = remoteTreeItems;                                                    // update remoteTree
    this.sort();

    remoteDirTree.reselectCurrentDirectory();                                       // select directory in remoteDirTree
    if (this.data.length)
      this.selection.select(0);                                                     // select first element in remoteTree
    this.mouseOver(null);
  },
  sort : function() {
    if (document.getElementById('remotename').getAttribute("sortDirection") &&
        document.getElementById('remotename').getAttribute("sortDirection") != "natural") {
      this.data.sort(compareRemoteName);
      if (document.getElementById('remotename').getAttribute("sortDirection") == "ascending")
        this.data.reverse();
    }
    if (document.getElementById('remotesize').getAttribute("sortDirection") && 
        document.getElementById('remotesize').getAttribute("sortDirection") != "natural") {
      this.data.sort(compareRemoteSize);
      if (document.getElementById('remotesize').getAttribute("sortDirection") == "ascending")
        this.data.reverse();
    }
    if (document.getElementById('remotedate').getAttribute("sortDirection") &&
        document.getElementById('remotedate').getAttribute("sortDirection") != "natural") {
      this.data.sort(compareRemoteDate);
      if (document.getElementById('remotedate').getAttribute("sortDirection") == "ascending")
        this.data.reverse();
    }
    if (document.getElementById('remotetype').getAttribute("sortDirection") &&
        document.getElementById('remotetype').getAttribute("sortDirection") != "natural") {
      this.data.sort(compareRemoteType);
      if (document.getElementById('remotetype').getAttribute("sortDirection") == "ascending")
        this.data.reverse();
    }
    if (document.getElementById('remoteattr').getAttribute("sortDirection") &&
        document.getElementById('remoteattr').getAttribute("sortDirection") != "natural") {
      this.data.sort(compareRemoteAttr);
      if (document.getElementById('remoteattr').getAttribute("sortDirection") == "ascending")
        this.data.reverse();
    }

    this.treebox.rowCountChanged(0, -this.rowCount);
    this.rowCount = this.data.length;
    this.treebox.rowCountChanged(0, this.rowCount);
  },

    // ************************************************* cut, copy, paste *****************************************************

  pasteFiles : new Array,
  oldParent : "",
  cut : function () {
    if (!isConnected || this.selection.count == 0)
      return;

    this.pasteFiles = new Array;
    this.oldParent = gRemotePath.value;
    for (var x = 0; x < this.rowCount; ++x)                                           // put files to be cut/copied in an array to be used when pasting
      if (this.selection.isSelected(x))
        this.pasteFiles.push(this.constructPath(gRemotePath.value, this.data[x].leafName));

    document.getElementById('remotePasteContext').setAttribute("disabled", false);    // enable pasting
  },
  paste : function() {
    if (!isConnected || this.pasteFiles.count == 0)
      return;

    var newParent = gRemotePath.value;
    if (newParent.indexOf(this.pasteFiles[0].path) != -1)                             // can't copy into a subdirectory of itself
      return;

    var prompt = true;
    var skipAll = false;

    for (var x = 0; x < this.pasteFiles.length; ++x) {
      var leafName = this.pasteFiles[x].substring(this.pasteFiles[x].lastIndexOf('/') + 1)
      var newPath = this.constructPath(newParent, leafName);

      var exists = false;
      for (var y = 0; y < this.data.length; ++y)
        if (this.data[y].leafName == leafName) {
          exists = true;
          break;
        }
      if (exists && skipAll)
        continue;
      if (exists && prompt) {
        var params = { response         : 0,
                       fileName         : leafName,
                       resume           : true,
                       replaceResume    : true,
                       timerEnable      : false };

        dialogOpen = true;
        window.openDialog("chrome://fireftp/content/confirmFile.xul","confirmFile",
          "width=425,height=90,chrome,modal=yes,dialog=yes,resizable=yes,centerscreen", params);
        dialogOpen = false;

        if (params.response == 2)
          prompt = false;
        else if (params.response == 3)
          continue;
        else if (params.response == 4 || params.response == 0)
          return;
        else if (params.response == 5) {
          skipAll = true;
          continue;
        }
      }

      addEventQueue("RNFR", this.pasteFiles[x]);
      addEventQueue("RNTO", newPath, x == this.pasteFiles.length - 1 && refreshMode ? "remoteTree.refresh()" : "");
    }

    
    var indexOfOldParent = remoteDirTree.indexOfPath(this.oldParent);
    var indexOfNewParent = remoteDirTree.indexOfPath(newParent);
    removeCacheEntry(this.oldParent);                                               // it's messy but it works
    removeCacheEntry(newParent);
    if (indexOfOldParent < indexOfNewParent) {
      if (indexOfNewParent != -1 && remoteDirTree.data[indexOfNewParent].open) {    // we reset the newParent's directory tree
        remoteDirTree.toggleOpenState(indexOfNewParent);
        remoteDirTree.data[indexOfNewParent].children = null;
      }
      if (indexOfOldParent != -1 && remoteDirTree.data[indexOfOldParent].open) {    // we reset the oldParent's directory tree
        remoteDirTree.toggleOpenState(indexOfOldParent);
        remoteDirTree.data[indexOfOldParent].children = null;
      }
      gRemotePath.value = this.oldParent;
    } else {                                                                        // else do it in opposite order
      if (indexOfOldParent != -1 && remoteDirTree.data[indexOfOldParent].open) {    // we reset the newParent's directory tree
        remoteDirTree.toggleOpenState(indexOfOldParent);
        remoteDirTree.data[indexOfOldParent].children = null;
      }
      if (indexOfNewParent != -1 && remoteDirTree.data[indexOfNewParent].open) {    // we reset the oldParent's directory tree
        remoteDirTree.toggleOpenState(indexOfNewParent);
        remoteDirTree.data[indexOfNewParent].children = null;
      }
      gRemotePath.value = newParent;
    }

    this.pasteFiles = new Array;
    document.getElementById('remotePasteContext').setAttribute("disabled", true);

    if (isReady)                                                                    // get the ball rollin' if it isn't already
      writeControl(eventQueue[0].cmd, eventQueue[0].parameter);
  },
  canDrop : function (aIndex, aOrientation) { return false; },
  drop    : function (aIndex, aOrientation) { }
};
